#ifndef GLOBAL_H
#define GLOBAL_H

#include "thread++.h"
#include "line.h"

// in main.yy
extern msn cfg_country;	// country code
extern msn cfg_area;	// area code
extern msn cfg_msn;	// configuration msn

class ctrl_connection : public connection {
  friend class mask_map;

protected:  
  enum { none, send1, poll_con } state;
  exponential_timeout<3000,3,2,30000> disc_timeout;
  packet sent;
  packet once; // discovery protocol
  
  /* messages a control connection can receive (mostly nops).  */
  void connect () {};
  void listen () {};
  void stop () {};
  void reset () {};
  void dump() {
     printf ("config connection\n");
  };
  
  /* wire functions */
  virtual packet send_packet ();
  virtual void receive_packet (const packet &p);
  virtual bool match (const packet &p) {
     return (p[0] & MSG_MASK) == MSG_SYN && (p[0] != MSG_SYN);
  };
  
  void send_ctrl_poll (const char *src, const char *dst, u8 reply);
public:
  ctrl_connection() { state = none; src_msn[0] = dst_msn[0] = 0; };
  
  void chat (const char *dst, const char *msg) {
     lock.lock ();
     sent.set (cfg_msn, dst, IEM_NONE);
     sent.put (MSG_CFG);
     sent.put (CFG_MSG);
     do { sent.put (*msg); } while (*msg++);
     tx_now ();
     lock.unlock ();
  };
  
  void send_rst (const packet &p) {
     clog (L_DEBUG, p.dst_msn, "sending RESET to %s\n", p.src_msn);
     once.set (p.dst_msn, p.src_msn, IEM_MIN);
     once.put (MSG_RST);
     once.put (0); // unspecified cause code
     tx_now ();
  };
  
  void cfg_poll (const char *dst) {
     log (L_JUNK, "config poll %s\n",dst);
     send_ctrl_poll (cfg_msn, dst, 0);
  };
  
  void cfg_poll_conn (const char *dst, u8 conn) {
  };
};

extern ctrl_connection control_connection; // all unfiltered packets go here

#endif

