%{
#include <config.h>

#include <stdio.h>
#include <string.h>
#include <assert.h>

extern "C" {
#ifdef HAVE_READLINE_H
#include <readline.h>
#include <history.h>
#else
#ifdef HAVE_READLINE_READLINE_H
#include <readline/readline.h>
#include <readline/history.h>
#endif
#endif
}

#include "main.h"

#define YY_INPUT(buf, res, len) res = yy_input(buf, len)

extern void yyerror (const char *msg);

static int yy_input (char *buf, int len)
{
  if (yyin == stdin)
    {
#ifdef HAVE_LIBREADLINE
      char *rl;
      
      assert (len >= 80); /*D*/ // FIXME
      
      for(;;)
        {
          rl = readline ("-> ");
          if (rl)
            {
              int len = strlen (rl);
              
              if (len)
                add_history (rl);
              
              strcpy (buf, rl);
              free (rl);
              buf[len++] = '\n';
              buf[len] = 0;
              return len;
            }
        }
#else
      printf ("-> "); fflush (stdout);
#endif
    }
  
  buf = fgets (buf, len, yyin);
  return buf ? strlen (buf) : YY_NULL;
}

%}

%option yylineno 

%x STR

MSN	[0-9]{1,40}
ALPHA	[a-z_][a-z_0-9-]*

		const char *yyfilename;

%%
		static char *string_buf_ptr;

"#"[^\n]*\n	;

modem(dev)?	return MODEMDEV;
ctrl(dev)?	return CTRLDEV;

connection	return CONNECTION;
connect|conn?	return CONNECT;
listen|lis	return LISTEN;
stop		return STOP;
quit		return QUIT;
list		return LIST;
info		return INFO;
poll		return POLL;
msg		return MSG;
cfg_msn		return CFG_MSN;
dump		return DUMP;
logopt|lo	return LOGOPT;
ident		return SERVER_IDENT;
alias		return ALIAS;

port		return PORT;
msn		return MSN;
src		return SRC;
dst		return DST;
tty_plug	return TTY_PLUG;

<<EOF>>		return QUIT;
;|\n		return EOL;

{MSN}		strcpy (yylval.str, yytext); return NUMERAL;
{ALPHA}		strcpy (yylval.str, yytext); return STRING;

[{}()]		return yytext[0];

!/.*		BEGIN(STR); string_buf_ptr = yylval.str; return MSG;
\"		BEGIN(STR); string_buf_ptr = yylval.str;
<STR>{
   \"|\n	BEGIN(INITIAL); *string_buf_ptr = '\0'; return STRING;
   \\\n		*string_buf_ptr++ = '\n';
   \\\"		*string_buf_ptr++ = '"';
   .		*string_buf_ptr++ = yytext[0];
}
[^;"[:space:]]+	strcpy (yylval.str, yytext); return STRING;

[[:space:]]	;

%%

// MUST stay here..
static int yywrap ()
{
  if (yyin != stdin)
    yyin = stdin;
  else
    return EOF;
  return 0;
}


